package cn.iocoder.yudao.module.wealth.controller.admin.commissionPayable;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.wealth.vo.commissionPayable.*;
import cn.iocoder.yudao.module.wealth.dal.dataobject.commissionPayable.CommissionPayableDO;
import cn.iocoder.yudao.module.wealth.convert.commissionPayable.CommissionPayableConvert;
import cn.iocoder.yudao.module.wealth.service.commissionPayable.CommissionPayableService;

@Validated
@RestController
@Api(tags = "管理后台 - 订单佣金应付款")
@RequestMapping("/ecw/commission-payable")
public class CommissionPayableController {

    @Resource
    private CommissionPayableService commissionPayableService;

    @PostMapping("/create")
    @ApiOperation("创建订单佣金应付款")
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payable:create')")
    public CommonResult<Long> createCommissionPayable(@Valid @RequestBody CommissionPayableCreateReqVO createReqVO) {
        return success(commissionPayableService.createCommissionPayable(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新订单佣金应付款")
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payable:update')")
    public CommonResult<Boolean> updateCommissionPayable(@Valid @RequestBody CommissionPayableUpdateReqVO updateReqVO) {
        commissionPayableService.updateCommissionPayable(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除订单佣金应付款")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payable:delete')")
    public CommonResult<Boolean> deleteCommissionPayable(@RequestParam("id") Long id) {
        commissionPayableService.deleteCommissionPayable(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得订单佣金应付款")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payable:query')")
    public CommonResult<CommissionPayableBackVO> getCommissionPayable(@RequestParam("id") Long id) {
        CommissionPayableDO commissionPayable = commissionPayableService.getCommissionPayable(id);
        return success(CommissionPayableConvert.INSTANCE.convert(commissionPayable));
    }

    @GetMapping("/list")
    @ApiOperation("获得订单佣金应付款列表")
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payable:query')")
    public CommonResult<List<CommissionPayableBackVO>> getCommissionPayableList(CommissionPayableQueryVO query) {
        List<CommissionPayableBackVO> list = commissionPayableService.getCommissionPayableList(query);
        return success(list);
    }

    @PostMapping("/page")
    @ApiOperation("获得订单佣金应付款分页")
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payable:query')")
    public CommonResult<PageResult<CommissionPayableBackVO>> getCommissionPayablePage(@RequestBody @Valid CommissionPayableQueryVO query, PageVO page) {
        PageResult<CommissionPayableBackVO> pageResult = commissionPayableService.getCommissionPayablePage(query, page);
        return success(pageResult);
    }


//    @GetMapping("/export-excel")
//    @ApiOperation("导出订单佣金应付款 Excel")
////    @PreAuthorize("@ss.hasPermission('ecw:commission-payable:export')")
//    @OperateLog(type = EXPORT)
//    public void exportCommissionPayableExcel(@Valid CommissionPayableQueryVO query,
//              HttpServletResponse response) throws IOException {
//        List<CommissionPayableDO> list = commissionPayableService.getCommissionPayableList(query);
//        // 导出 Excel
//        List<CommissionPayableBackVO> datas = CommissionPayableConvert.INSTANCE.convertList(list);
//        ExcelUtils.write(response, "订单佣金应付款.xls", "数据", CommissionPayableBackVO.class, datas);
//    }
}
