package cn.iocoder.yudao.module.wealth.controller.admin.commissionPayment;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.convert.paymentApproval.PaymentApprovalConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.paymentApproval.PaymentApprovalDO;
import cn.iocoder.yudao.module.wealth.service.commissionPayment.CommissionPaymentService;
import cn.iocoder.yudao.module.wealth.service.paymentApproval.PaymentApprovalService;
import cn.iocoder.yudao.module.wealth.vo.commissionPayment.CommissionPaymentBackVO;
import cn.iocoder.yudao.module.wealth.vo.commissionPayment.CommissionPaymentCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.commissionPayment.CommissionPaymentQueryVO;
import cn.iocoder.yudao.module.wealth.vo.commissionPayment.CommissionPaymentUpdateReqVO;
import cn.iocoder.yudao.module.wealth.vo.payment.FinancePaymentCancelVo;
import cn.iocoder.yudao.module.wealth.vo.payment.FinancePaymentVo;
import cn.iocoder.yudao.module.wealth.vo.paymentApproval.PaymentApprovalBackVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 佣金付款单")
@RequestMapping("/ecw/commission-payment")
public class CommissionPaymentController {

    @Resource
    private CommissionPaymentService commissionPaymentService;
    @Resource
    private PaymentApprovalService paymentApprovalService;

    @PostMapping("/create")
    @ApiOperation("创建佣金付款单")
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payment:create')")
    public CommonResult<Long> createCommissionPayment(@Valid @RequestBody CommissionPaymentCreateReqVO createReqVO) {
        return success(commissionPaymentService.createCommissionPayment(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新佣金付款单")
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payment:update')")
    public CommonResult<Boolean> updateCommissionPayment(@Valid @RequestBody CommissionPaymentUpdateReqVO updateReqVO) {
        commissionPaymentService.updateCommissionPayment(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除佣金付款单")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payment:delete')")
    public CommonResult<Boolean> deleteCommissionPayment(@RequestParam("id") Long id) {
        commissionPaymentService.deleteCommissionPayment(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得佣金付款单")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payment:query')")
    public CommonResult<CommissionPaymentBackVO> getCommissionPayment(@RequestParam("id") Long id) {
        return success(commissionPaymentService.info(id));
//        CommissionPaymentDO commissionPayment = commissionPaymentService.getCommissionPayment(id);
//        return success(CommissionPaymentConvert.INSTANCE.convert(commissionPayment));
    }

    @GetMapping("/list")
    @ApiOperation("获得佣金付款单列表")
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payment:query')")
    public CommonResult<List<CommissionPaymentBackVO>> getCommissionPaymentList(CommissionPaymentQueryVO query) {
        List<CommissionPaymentBackVO> list = commissionPaymentService.getCommissionPaymentList(query);
        return success(list);
    }

    @PostMapping("/page")
    @ApiOperation("获得佣金付款单分页")
//    @PreAuthorize("@ss.hasPermission('ecw:commission-payment:query')")
    public CommonResult<PageResult<CommissionPaymentBackVO>> getCommissionPaymentPage(@RequestBody @Valid CommissionPaymentQueryVO query, PageVO page) {
        PageResult<CommissionPaymentBackVO> pageResult = commissionPaymentService.getCommissionPaymentPage(query, page);
        return success(pageResult);
    }

    @GetMapping("/getPaymentApprove")
    @ApiOperation("根据流程主键获取付款单流程审核信息")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<PaymentApprovalBackVO> getPaymentApprove(@RequestParam("id") Long id) {
        PaymentApprovalDO paymentApprovalDO = paymentApprovalService.getPaymentApproval(id);
        return success(PaymentApprovalConvert.INSTANCE.convert(paymentApprovalDO));
    }

    @PostMapping("/cancelFinancePaymentApproval")
    @ApiOperation("取消付款单审核")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinancePaymentApproval(@RequestBody FinancePaymentCancelVo createReqVO) {
        commissionPaymentService.cancelFinancePaymentApproval(createReqVO);
        return success(true);
    }

    @PostMapping("/verifyCancel")
    @ApiOperation("反审核")
    /*//@PreAuthorize("@ss.hasPermission('ecw:receipt:query')")*/
    public CommonResult<Boolean> verifyPaymentCancel(@RequestBody FinancePaymentVo financePaymentVo) {
        commissionPaymentService.verifyPaymentCancel(financePaymentVo);
        return success(true);
    }


    @PostMapping("/cancelFinancePaymentApprovalNo")
    @ApiOperation("取消付款单反审核")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinancePaymentApprovalNo(@RequestBody FinancePaymentCancelVo createReqVO) {
        commissionPaymentService.cancelFinancePaymentApprovalNo(createReqVO);
        return success(true);
    }

    @PostMapping("/verification")
    @ApiOperation("付款单核销")
    /*//@PreAuthorize("@ss.hasPermission('ecw:receipt:query')")*/
    public CommonResult<Boolean> verification(@RequestBody FinancePaymentVo financePaymentVo) {
        commissionPaymentService.verification(financePaymentVo);
        return success(true);
    }


    @PostMapping("/cancelFinancePaymentWriteOff")
    @ApiOperation("取消付款单核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinancePaymentWriteOff(@RequestBody FinancePaymentCancelVo createReqVO) {
        commissionPaymentService.cancelFinancePaymentWriteOff(createReqVO);
        return success(true);
    }

    @PostMapping("/verificationCancel")
    @ApiOperation("付款单反核销")
    /*//@PreAuthorize("@ss.hasPermission('ecw:receipt:query')")*/
    public CommonResult<Boolean> verificationCancel(@RequestBody FinancePaymentVo financePaymentVo) {
        commissionPaymentService.verificationCancel(financePaymentVo);
        return success(true);
    }

    @PostMapping("/cancelFinancePaymentWriteOffNo")
    @ApiOperation("取消付款单反核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinancePaymentWriteOffNo(@RequestBody FinancePaymentCancelVo createReqVO) {
        commissionPaymentService.cancelFinancePaymentWriteOffNo(createReqVO);
        return success(true);
    }
}
