package cn.iocoder.yudao.module.wealth.controller.admin.payable;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.convert.payable.PayableConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payable.PayableDO;
import cn.iocoder.yudao.module.wealth.service.payable.PayableService;
import cn.iocoder.yudao.module.wealth.vo.payable.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 应付款")
@RequestMapping("/ecw/payable")
public class PayableController {

    @Resource
    private PayableService payableService;

    @PostMapping("/create")
    @ApiOperation("创建应付款")
    //@PreAuthorize("@ss.hasPermission('ecw:payable:create')")
    public CommonResult<Long> createPayable(@Valid @RequestBody PayableCreateReqVO createReqVO) {
        String s="";
        return success(payableService.createPayable(createReqVO));
    }


    @PutMapping("/update")
    @ApiOperation("更新应付款")
    //@PreAuthorize("@ss.hasPermission('ecw:payable:update')")
    public CommonResult<Boolean> updatePayable(@Valid @RequestBody PayableUpdateReqVO updateReqVO) {
        payableService.updatePayable(updateReqVO);
        return success(true);
    }


    @DeleteMapping("/delete")
    @ApiOperation("删除应付款")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:payable:delete')")
    public CommonResult<Boolean> deletePayable(@RequestParam("id") Long id) {
        payableService.deletePayable(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得应付款")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:payable:query')")
    public CommonResult<PayableBackVO> getPayable(@RequestParam("id") Long id) {
        PayableDO payable = payableService.getPayable(id);
        return success(PayableConvert.INSTANCE.convert(payable));
    }

    @GetMapping("/list")
    @ApiOperation("获得应付款列表 ")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:payable:query')")
    public CommonResult<List<PayableBackVO>> getPayableList(@RequestParam("ids") Collection<Long> ids) {
        List<PayableDO> list = payableService.getPayableList(ids);
        return success(PayableConvert.INSTANCE.convertList(list));
    }

    @PostMapping("/page")
    @ApiOperation("应付款分页列表，添加供应商未付款项列表 两个用这一个查询")
    //@PreAuthorize("@ss.hasPermission('ecw:payable:query')")
    public CommonResult<PageResult<PayableOrderBackVo>> getPayablePage(@RequestBody @Valid PayableQueryVO query, PageVO page) {
        PageResult<PayableOrderBackVo> result = payableService.getPayablePage(query, page);
        return success(result);
    }

    /*@GetMapping("/export-excel")
    @ApiOperation("导出应付款 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:payable:export')")
    @OperateLog(type = EXPORT)
    public void exportPayableExcel(@Valid PayableQueryVO query,
                                   HttpServletResponse response) throws IOException {
        List<PayableDO> list = payableService.getPayableList(query);
        // 导出 Excel
        List<PayableBackVO> datas = PayableConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "应付款.xls", "数据", PayableBackVO.class, datas);
    }*/
}
