package cn.iocoder.yudao.module.wealth.controller.admin.payment;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.io.FileModel;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import cn.iocoder.yudao.module.wealth.convert.payable.PayableConvert;
import cn.iocoder.yudao.module.wealth.convert.payment.PaymentConvert;
import cn.iocoder.yudao.module.wealth.convert.paymentApproval.PaymentApprovalConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payable.PayableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.payment.PaymentDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.paymentApproval.PaymentApprovalDO;
import cn.iocoder.yudao.module.wealth.service.payable.PayableService;
import cn.iocoder.yudao.module.wealth.service.payment.PaymentService;
import cn.iocoder.yudao.module.wealth.service.paymentApproval.PaymentApprovalService;
import cn.iocoder.yudao.module.wealth.vo.payable.PayableBackVO;
import cn.iocoder.yudao.module.wealth.vo.payment.*;
import cn.iocoder.yudao.module.wealth.vo.paymentApproval.PaymentApprovalBackVO;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptCancelVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 付款单")
@RequestMapping("/ecw/payment")
public class PaymentController {

    @Resource
    private PaymentService paymentService;

    @Resource
    private PayableService payableService;

    @Resource
    private PaymentApprovalService paymentApprovalService;

    /*
    */
    @Autowired
    private AdminUserService adminUserService;


    @PostMapping("/create")
    @ApiOperation("创建付款单")
    @Idempotent(timeout = 5)
    //@PreAuthorize("@ss.hasPermission('ecw:payment:create')")
    public CommonResult<Long> createPayment(@Valid @RequestBody PaymentCreateReqVO createReqVO) {
        return success(paymentService.createPayment(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新付款单")
    @Idempotent(timeout = 5)
    //@PreAuthorize("@ss.hasPermission('ecw:payment:update')")
    public CommonResult<Boolean> updatePayment(@Valid @RequestBody PaymentUpdateReqVO updateReqVO) {
        paymentService.updatePayment(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除付款单")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:payment:delete')")
    public CommonResult<Boolean> deletePayment(@RequestParam("id") Long id) {
        paymentService.deletePayment(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得付款单详细信息")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<PaymentBackVO> getPayment(@RequestParam("id") Long id) {
        List<AdminUserDO> ListUser = adminUserService.getUsersList();
        Map<Long, String> userMap =
                ListUser.stream().collect(
                        Collectors.toMap((AdminUserDO::getId),
                                AdminUserDO::getNickname));

        //修改问题点
        PaymentDO payment = paymentService.getPayment(id);
        if (payment.getFkren()!=null&& payment.getFkren().length() > 0) {
            payment.setFkren(userMap.get(Long.parseLong(payment.getFkren())));
        } else {
            payment.setFkren("");
        }
        // 添加附件列表字段逻辑
        List<FileModel> fileList =
                JSONUtil.toList(new JSONArray(payment.getFilepath()),
                        FileModel.class);

        PaymentBackVO vo= PaymentConvert.INSTANCE.convert(payment);
        vo.setFilelist(fileList);
        //end  添加附件列表字段逻辑
        return success(vo);
    }

    @GetMapping("/getPaymentItem")
    @ApiOperation("获得付款单付款明细")
    @ApiImplicitParam(name = "id", value = "付款单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<PayableBackVO>> getPaymentItem(@RequestParam("id") Long id) {
        LambdaQueryWrapper<PayableDO> lambdaQueryWrapper = new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(PayableDO::getPaymentId, id);
        return success(PayableConvert.INSTANCE.convertList(payableService.list(lambdaQueryWrapper)));
    }


    @PostMapping("/page")
    @ApiOperation("获得付款单分页")
    //@PreAuthorize("@ss.hasPermission('ecw:payment:query')")
    public CommonResult<PageResult<PaymentBackVO>> getPaymentPage(@RequestBody @Valid PaymentQueryVO query, PageVO page) {
        PageResult<PaymentDO> pageResult = paymentService.getPaymentPage(query, page);
        return success(PaymentConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/getPaymentApprove")
    @ApiOperation("根据流程主键获取付款单流程审核信息")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<PaymentApprovalBackVO> getPaymentApprove(@RequestParam("id") Long id) {
        PaymentApprovalDO paymentApprovalDO = paymentApprovalService.getPaymentApproval(id);
        return success(PaymentApprovalConvert.INSTANCE.convert(paymentApprovalDO));
    }

    @PostMapping("/cancelFinancePaymentApproval")
    @ApiOperation("取消付款单审核")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinancePaymentApproval(@RequestBody FinancePaymentCancelVo createReqVO) {
        paymentService.cancelFinancePaymentApproval(createReqVO);
        return success(true);
    }

    @PostMapping("/verifyCancel")
    @ApiOperation("付款单反审核")
    @Idempotent(timeout = 5)
    /*//@PreAuthorize("@ss.hasPermission('ecw:receipt:query')")*/ public CommonResult<Boolean> verifyPaymentCancel(@RequestBody FinancePaymentVo financePaymentVo) {
        paymentService.verifyPaymentCancel(financePaymentVo);
        return success(true);
    }

    @PostMapping("/cancelFinancePaymentApprovalNo")
    @ApiOperation("取消付款单反审核")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinancePaymentApprovalNo(@RequestBody FinancePaymentCancelVo createReqVO) {
        paymentService.cancelFinancePaymentApprovalNo(createReqVO);
        return success(true);
    }

    @PostMapping("/verification")
    @ApiOperation("付款单核销")
    @Idempotent(timeout = 5)
    /*//@PreAuthorize("@ss.hasPermission('ecw:receipt:query')")*/
    public CommonResult<Boolean> verification(@RequestBody FinancePaymentVo financePaymentVo) {
        //核销
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        financePaymentVo.setFkren(loginUserId.toString());
        paymentService.verification(financePaymentVo);
        return success(true);
    }

    @PostMapping("/cancelFinancePaymentWriteOff")
    @ApiOperation("取消付款单核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinancePaymentWriteOff(@RequestBody FinancePaymentCancelVo createReqVO) {
        paymentService.cancelFinancePaymentWriteOff(createReqVO);
        return success(true);
    }


    @PostMapping("/verificationCancel")
    @ApiOperation("付款单反核销")
    @Idempotent(timeout = 5)
    /*//@PreAuthorize("@ss.hasPermission('ecw:receipt:query')")*/ public CommonResult<Boolean> verificationCancel(@RequestBody FinancePaymentVo financePaymentVo) {
        paymentService.verificationCancel(financePaymentVo);
        return success(true);
    }

    @PostMapping("/cancelFinancePaymentWriteOffNo")
    @ApiOperation("取消付款单反核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinancePaymentWriteOffNo(@RequestBody FinancePaymentCancelVo createReqVO) {
        paymentService.cancelFinancePaymentWriteOffNo(createReqVO);
        return success(true);
    }
}
