package cn.iocoder.yudao.module.wealth.controller.admin.receiptInvoice;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.service.receiptInvoice.ReceiptInvoiceService;
import cn.iocoder.yudao.module.wealth.vo.receiptInvoice.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 收款单开票信息")
@RequestMapping("/ecw/receipt-invoice")
public class ReceiptInvoiceController {

    @Resource
    private ReceiptInvoiceService receiptInvoiceService;

    @PostMapping("/create")
    @ApiOperation("申请开票")
    public CommonResult<Long> createReceiptInvoice(@Valid @RequestBody InvoiceCreateReqVO createReqVO) {
        return success(receiptInvoiceService.createReceiptInvoice(createReqVO));
    }

    @PostMapping("/update/data")
    @ApiOperation("更新收款单开票资料")
    public CommonResult<Boolean> updateReceiptInvoiceData(@Valid @RequestBody ReceiptInvoiceUpdateDataReqVO updateReqVO) {
        receiptInvoiceService.updateReceiptInvoiceData(updateReqVO);
        return success(true);
    }
    @PostMapping("/update/info")
    @ApiOperation("更新收款单开票信息")
    public CommonResult<Boolean> updateReceiptInvoiceInfo(@Valid @RequestBody ReceiptInvoiceUpdateInfoReqVO updateReqVO) {
        receiptInvoiceService.updateReceiptInvoiceInfo(updateReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得收款单开票信息")
    public CommonResult<ReceiptInvoiceDetailVO> getReceiptInvoiceDetail(@RequestParam("id") Long id) {
        ReceiptInvoiceDetailVO receiptInvoiceDetail = receiptInvoiceService.getReceiptInvoiceDetail(id);
        return success(receiptInvoiceDetail);
    }

    @GetMapping("/receipt/info/get")
    @ApiOperation("获得收款单开票信息")
    public CommonResult<ReceiptInvoiceDetailVO> getReceiptDetailForCreateInvoice(@RequestParam("receiptId") Long receiptId) {
        ReceiptInvoiceDetailVO receiptInvoiceDetail = receiptInvoiceService.getReceiptDetailForCreateInvoice(receiptId);
        return success(receiptInvoiceDetail);
    }

    @PostMapping("/page")
    @ApiOperation("获得收款单开票信息分页")
    public CommonResult<PageResult<ReceiptInvoicePageBackVO>> getReceiptInvoicePage(@RequestBody @Valid ReceiptInvoiceQueryVO query, PageVO page) {
        PageResult<ReceiptInvoicePageBackVO> pageResult = receiptInvoiceService.getReceiptInvoicePage(query, page);
        return success(pageResult);
    }

    @PostMapping("/reject")
    @ApiOperation("驳回开票信息")
    public CommonResult<Boolean> rejectInvoiceInfo(@RequestBody @Valid ReceiptInvoiceOperateReqVO reqVO) {
        Boolean res = receiptInvoiceService.rejectInvoiceInfo(reqVO);
        return success(res);
    }

    @PostMapping("/cancel")
    @ApiOperation("取消开票")
    public CommonResult<Boolean> cancelInvoice(@RequestBody @Valid ReceiptInvoiceOperateReqVO reqVO) {
        Boolean res = receiptInvoiceService.cancelInvoice(reqVO);
        return success(res);
    }

    @PostMapping("/delete")
    @ApiOperation("删除开票")
    public CommonResult<Boolean> deleteInvoiceInfo(@RequestBody @Valid ReceiptInvoiceOperateReqVO reqVO) {
        Boolean res = receiptInvoiceService.deleteInvoiceInfo(reqVO);
        return success(res);
    }



    //@GetMapping("/export-excel")
    //@ApiOperation("导出收款单开票信息 Excel")
    //@OperateLog(type = EXPORT)
    //public void exportReceiptInvoiceExcel(@Valid ReceiptInvoiceQueryVO query,
    //                                      HttpServletResponse response) throws IOException {
    //    List<ReceiptInvoiceDO> list = receiptInvoiceService.getReceiptInvoiceList(query);
    //    // 导出 Excel
    //    List<ReceiptInvoicePageBackVO> datas = ReceiptInvoiceConvert.INSTANCE.convertList(list);
    //    ExcelUtils.write(response, "收款单开票信息.xls", "数据", ReceiptInvoicePageBackVO.class, datas);
    //}
}
