package cn.iocoder.yudao.module.wealth.controller.admin.receivable;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.wealth.convert.receivableDiscount.ReceivableDiscountConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivableDiscount.ReceivableDiscountDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receivableDiscount.ReceivableDiscountMapper;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.vo.receivable.*;
import cn.iocoder.yudao.module.wealth.vo.receivableDiscount.ReceivableDiscountBackVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 应收款")
@RequestMapping("/ecw/receivable")
public class ReceivableController {

    @Resource
    private ReceivableService receivableService;

    @Resource
    private ReceivableDiscountMapper receivableDiscountMapper;


    @PostMapping("/page")
    @ApiOperation("应收款分页列表")
    //@PreAuthorize("@ss.hasPermission('ecw:receivable:query')")
    public CommonResult<PageResult<ReceivableBackVO>> getReceivablePage(@RequestBody @Valid ReceivableQueryVO query, PageVO page) {
        PageResult<ReceivableBackVO> pageResult = receivableService.receivablePage(query, page);
        return success(pageResult);
    }

    @PostMapping("/page/amount")
    @ApiOperation("应收款分页条件总金额合计")
    public CommonResult<List<WealthMoneyAmountVO>> getReceivablePage(@RequestBody @Valid ReceivableQueryVO query) {
        List<WealthMoneyAmountVO> pageAmount = receivableService.receivablePageAmount(query);
        return success(pageAmount);
    }

    @GetMapping("/getReceivableByOrderId")
    @ApiOperation("根据订单ID获取所有未加入收款单的应收明细")
    public CommonResult<List<ReceivableBackVO>> getReceivableByOrderId(@RequestParam("orderId") Long orderId) {
        return success(receivableService.getReceivableByOrderId(orderId));
    }

    @GetMapping("/get")
    @ApiOperation("获得应收款")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:receivable:query')")
    public CommonResult<ReceivableBackVO> getReceivable(@RequestParam("id") Long id) {
        return success(receivableService.getReceivableBackVo(id));
    }


    @GetMapping("/getReceivableListByOrderId")
    @ApiOperation("根据订单ID获得应收款列表")
    @ApiImplicitParam(name = "id", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<ReceivableBackVO>> getReceivableListByOrderId(
            @RequestParam("id") Long id) {
        return success(receivableService.getReceivableListByOrderId(id));
    }

    @GetMapping("/getFirstReceivableListByOrderId")
    @ApiOperation("根据订单ID获得需要预付应收款列表")
    @ApiImplicitParam(name = "id", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<ReceivableBackVO>> getFirstReceivableListByOrderId(@RequestParam("id") Long id) {
        return success(receivableService.getFirstReceivableListByOrderId(id));
    }

    @PostMapping("/updateReceivableDiscountById")
    @ApiOperation("根据应收ID更新应收优惠信息")
    public CommonResult<Boolean> updateReceivableDiscountById(@Valid @RequestBody ReceivableUpdateReqVO receivableUpdateReqVO) {
        receivableService.updateReceivableDiscountById(receivableUpdateReqVO);
        return success(true);
    }

    @GetMapping("/getReceivableDiscountLogById")
    @ApiOperation("得到应收优惠的最后操作人")
    @ApiImplicitParam(name = "id", value = "应收ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<ReceivableDiscountBackVO> updateReceivableDiscountById(@RequestParam("id") Long id) {
        LambdaQueryWrapper<ReceivableDiscountDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceivableDiscountDO::getReceivableId, id);
        lambdaQueryWrapper.orderByDesc(ReceivableDiscountDO::getCreateTime);
        lambdaQueryWrapper.last("limit 1");
        return success(ReceivableDiscountConvert.INSTANCE.convert(receivableDiscountMapper.selectOne(lambdaQueryWrapper)));
    }

    @PostMapping("/page/costDetail")
    @ApiOperation("财务-费用明细")
    public CommonResult<CostDetailBackVO> getCostDetailPage(@Valid @RequestBody CostDetailPageQueryVO query, PageVO page) {
        CostDetailBackVO pageResult = receivableService.getCostDetailPage(query, page);
        return success(pageResult);
    }

    @PostMapping("/batch-gen-receipt/page")
    @ApiOperation("财务-批量生成收款单列表")
    public CommonResult<PageResult<BatchGenReceiptPageVO>> batchGenReceiptPage(@Valid @RequestBody BatchGenReceiptPageQueryVO query, PageVO page) {
        return success(receivableService.batchGenReceiptPage(query, page));
    }

    @PostMapping("/batch-gen-receipt/receivable/page")
    @ApiOperation("财务-批量生成收款单费用明细列表")
    public CommonResult<ReceivablePageResult<BatchGenReceiptReceivablePageVO>> batchGenReceiptReceivablePage(@Valid @RequestBody BatchGenReceiptReceivablePageQueryVO query, PageVO page) {
        return success(receivableService.batchGenReceiptReceivablePage(query, page));
    }

    @GetMapping("/getListForReceiptItem")
    @ApiOperation("财务-创建收款明细获取应收明细")
    public CommonResult<List<ReceivableInItemVO>> getListForCreateReceiptItem(Long receiptId) {
        return success(receivableService.getListForCreateReceiptItem(receiptId));
    }

    @GetMapping("/getReceivableByReceiptItem")
    @ApiOperation("财务-根据收款明细获取应收明细")
    public CommonResult<List<ReceivableInItemVO>> getReceivableByReceiptItem(Long receiptItemId) {
        return success(receivableService.getReceivableByReceiptItem(receiptItemId));
    }

    @PostMapping("/box/report")
    @ApiOperation("财务-应收报表")
    @Idempotent(timeout = 5)
    public CommonResult<ReceivableBoxReportVO> getReceivableBoxReport(@Valid @RequestBody ReceivableBoxReportQueryVO query, PageVO page) {
        return success(receivableService.getReceivableBoxReport(query, page));
    }
}
